#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os.path
import dsmz


class BacDive(dsmz.API):
    def __init__(self, user, password, url_prefix='https://bacdive.dsmz.de/api/bacdive'):
        dsmz.API.__init__(self, user, password, url_prefix)

    def all_links(self, fmt=dsmz.Format.JSON):
        return self.request_service('/bacdive_id/', fmt)

    def bacdive_id(self, bdid, fmt=dsmz.Format.JSON):
        return self.request_service('/bacdive_id/%s/' % bdid, fmt)

    @staticmethod
    def _build_taxon_service(genus, species=None, subspecies=None):
        if species is None:
            if subspecies is not None:
                raise RuntimeError('subspecies epithet specified but no species epithet')
            return '/taxon/%s/' % genus
        if subspecies is None:
            return '/taxon/%s/%s/' % (genus, species)
        return '/taxon/%s/%s/%s/' % (genus, species, subspecies)

    def taxon(self, genus, species=None, subspecies=None, fmt=dsmz.Format.JSON):
        return self.request_service(BacDive._build_taxon_service(genus, species, subspecies), fmt)


class BacDiveCLI(dsmz.CLI):
    def __init__(self):
        dsmz.CLI.__init__(self, description='CLI client for BacDive services')
        self.query_group = self.add_mutually_exclusive_group()
        self.query_group.add_argument('-t', '--taxon', action='store_const', const='taxon', dest='query', default='taxon', help='query by taxon name (genus [species [subspecies]]) <>')
        self.query_group.add_argument('-i', '--bacdive-id', action='store_const', const='bacdive_id', dest='query', default='taxon', help='query by bacdive_id')
        self.query_group.add_argument('-a', '--all-entries', action='store_const', const='all_entries', dest='query', default='taxon', help='query all entries')

    def default_url_prefix(self):
        return 'https://bacdive.dsmz.de/api/bacdive'

    def run(self):
        args = self.get_args()
        bacdive = BacDive(args.user, args.password, url_prefix=args.url_prefix)
        if args.query == 'taxon':
            BacDiveCLI.taxon(bacdive, args.fmt, args.outdir, args.terms)
        elif args.query == 'bacdive_id':
            BacDiveCLI.bacdive_id(bacdive, args.fmt, args.outdir, args.terms)
        elif args.query == 'all_entries':
            BacDiveCLI.all_entries(bacdive, args.fmt, args.outdir, args.terms)

    @staticmethod
    def bacdive_id(bacdive, fmt, outdir, terms):
        bdid = terms.pop(0)
        response = dsmz.CLI._response_or_log(bacdive.bacdive_id(bdid, fmt))
        dsmz.CLI._save_response_content(response, os.path.join(outdir, bdid + fmt.ext))

    @staticmethod
    def taxon(bacdive, fmt, outdir, terms):
        genus = terms.pop(0)
        species = dsmz.CLI._pop_or_None(terms)
        subspecies = dsmz.CLI._pop_or_None(terms)
        response = dsmz.CLI._response_or_log(bacdive.taxon(genus, species, subspecies))
        url_prefix = bacdive._build_service_url('/bacdive_id/')
        while response is not None:
            j = response.json()
            for o in j['results']:
                url = o['url']
                bid = url.replace(url_prefix, '').replace('/', '')
                r = dsmz.CLI._response_or_log(bacdive.request_raw_url(url, fmt=fmt))
                dsmz.CLI._save_response_content(r, os.path.join(outdir, bid + fmt.ext))
            next_page_url = j['next']
            if next_page_url is None:
                break
            response = dsmz.CLI._response_or_log(bacdive.request_raw_url(next_page_url))

    @staticmethod
    def all_entries(bacdive, fmt, outdir, terms):
        response = dsmz.CLI._response_or_log(bacdive.all_links())
        page = 1
        url_prefix = bacdive._build_service_url('/bacdive_id/')
        while response is not None:
            page_dir = os.path.join(outdir, 'page-%03d' % page)
            os.makedirs(page_dir, exist_ok=True)
            j = response.json()
            for o in j['results']:
                url = o['url']
                bid = url.replace(url_prefix, '').replace('/', '')
                r = dsmz.CLI._response_or_log(bacdive.request_raw_url(url, fmt=fmt))
                dsmz.CLI._save_response_content(r, os.path.join(page_dir, bid + fmt.ext))
            next_page_url = j['next']
            if next_page_url is None:
                break
            page += 1
            response = dsmz.CLI._response_or_log(bacdive.request_raw_url(next_page_url))


if __name__ == '__main__':
    BacDiveCLI().run()
